@@ -1,4 +1,4 @@
-/* $Id: gdict-app.c,v 1.130 2005/05/25 15:01:40 vnoel Exp $ */
+/* $Id: gdict-app.c,v 1.143 2005/07/05 16:01:12 vnoel Exp $ */
 
 /*
  *  Mike Hughes <mfh@psilord.com>
@@ -16,9 +16,13 @@
 #include <config.h>
 #endif
 
-#include <gtk/gtk.h>
-#include <glib/gi18n.h>
+#include <string.h>
+#include <libgnomeprint/gnome-print-config.h>
+#include <libgnomeprint/gnome-print-job.h>
+#include <libgnomeprintui/gnome-print-dialog.h>
 #include <libgnomeui/gnome-entry.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 #include "dict.h"
 #include "gdict-app.h"
 #include "gdict-about.h"
@@ -28,18 +32,17 @@
 #include "gdict-speller.h"
 #include "gdict-utils.h"
 #include "gdict-web-search.h"
+#include "gdict-entry.h"
 
-#include <libgnomeprint/gnome-print-config.h>
-#include <libgnomeprint/gnome-print-job.h>
-#include <libgnomeprintui/gnome-print-dialog.h>
-
-#define GDICT_DEFAULT_WIDTH  450
+#define GDICT_DEFAULT_WIDTH  400
 #define GDICT_DEFAULT_HEIGHT 330
 
 static GObjectClass *parent_class;
-gboolean  gail_loaded = FALSE;
+gboolean gail_loaded = FALSE;
 GSList *gdict_windows = NULL;
 
+static void gdict_menu_item_set_sensitive (GDictWindow *gdict, char *path, gboolean sensitive);
+
 enum
 {
   TARGET_STRING,
@@ -49,14 +52,25 @@ enum
   TARGET_TEXT_BUFFER_CONTENTS
 };
 
+static gboolean
+gdict_is_pattern (gchar *text) 
+{
+  if (strpbrk (text, "*|{}()[]"))
+    return TRUE;
+  else
+    return FALSE;
+}
+
 /* gdict_init_context
  * Initialises the context object with information on the current server
- * Returns 0 on success, -1 if the server could not be found
+ * Returns 0 on success, -1 if the server could not be found, -2 for
+ * invalid parameter.
  */
+
 gint
 gdict_init_context (GDictWindow *gdict) 
 {
-  g_return_if_fail (GDICT_IS_WINDOW (gdict));
+  g_return_val_if_fail (GDICT_IS_WINDOW (gdict), -2);
 
   if (gdict->context) 
     dict_context_destroy (gdict->context);
@@ -91,37 +105,6 @@ gdict_socket_error_cb (GtkWidget *widget, const gchar *message, gpointer data)
   gdk_window_set_cursor (GTK_WIDGET(gdict)->window, NULL);
 }
 
-gint
-gdict_lookup_spelling (GDictWindow *gdict, gchar *text, gboolean pattern) 
-{
-  GDictSpeller *speller = GDICT_SPELLER(gdict->speller);
-  GtkWidget *check;
-
-  g_return_val_if_fail (text != NULL, 0);
-
-  if (pattern) 
-    speller->strat = "re";
-  else
-    speller->strat = gdict->pref->dfl_strat;
-
-  check = gtk_ui_manager_get_widget (gdict->ui, "/MenuBar/ViewMenu/ViewSpellings");
-  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (check), TRUE);
-  
-  if (gdict_speller_lookup (gdict, speller, text) == -1)
-    return -1;
-
-  return 0;
-}
-
-static gboolean
-is_pattern (gchar *text) 
-{
-  if (strpbrk (text, "*|{}()[]"))
-    return TRUE;
-  else
-    return FALSE;
-}
-
 void
 gdict_not_online (GDictWindow *gdict) 
 {
@@ -145,23 +128,13 @@ void
 gdict_lookup_definition (GDictWindow *gdict, gchar *text) 
 {
   gint retval;
-  gchar *word_entry_text;
 
 	if (text == NULL)
 		return;
 
-  /* If needed update the the word_entry */
-  word_entry_text = gtk_editable_get_chars (GTK_EDITABLE (gdict->word_entry), 0, -1);
-  if ((word_entry_text == NULL) ||
-      (*word_entry_text == 0)   ||
-      (strcmp (word_entry_text, text) != 0)) {
-    gnome_entry_prepend_history (GNOME_ENTRY (gdict->gnome_word_entry), 1, text);
-    gtk_entry_set_text (GTK_ENTRY (gdict->word_entry), text);
-  }
+	gtk_entry_set_text (GTK_ENTRY (gdict->word_entry), text);	
 
-  g_free (word_entry_text);
-
-  if (is_pattern (text))
+  if (gdict_is_pattern (text))
     retval = gdict_lookup_spelling (gdict, text, TRUE);
   else {
     retval = gdict_defbox_lookup (gdict->defbox, text);
@@ -173,6 +146,28 @@ gdict_lookup_definition (GDictWindow *gdict, gchar *text)
     gdict_not_online (gdict);
 }
 
+gint
+gdict_lookup_spelling (GDictWindow *gdict, gchar *text, gboolean pattern) 
+{
+  GDictSpeller *speller = GDICT_SPELLER(gdict->speller);
+  GtkWidget *check;
+
+  g_return_val_if_fail (text != NULL, 0);
+
+  if (pattern) 
+    speller->strat = "re";
+  else
+    speller->strat = gdict->pref->dfl_strat;
+
+  check = gtk_ui_manager_get_widget (gdict->ui, "/MenuBar/ViewMenu/ViewSpellings");
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (check), TRUE);
+  
+  if (gdict_speller_lookup (gdict, speller, text) == -1)
+    return -1;
+
+  return 0;
+}
+
 void
 gdict_lookup_web (GDictWindow *gdict, gchar *text)
 {
@@ -231,25 +226,6 @@ gdict_lookup_cb (GtkAction *action, GtkWidget *widget)
   g_free (text);
 }
 
-static void
-gdict_menu_item_set_sensitive (GDictWindow *gdict, char *path, gboolean sensitive)
-{
-  GtkAction *action;
-
-  g_return_if_fail (path);
-  action = gtk_ui_manager_get_action (gdict->ui, path);
-  gtk_action_set_sensitive (action, sensitive);
-}
-
-void
-gdict_menus_set_sensitive (GDictWindow *gdict, gboolean found_definition)
-{
-  gdict_menu_item_set_sensitive (gdict, "/MenuBar/DictionaryMenu/Print", found_definition);
-  gdict_menu_item_set_sensitive (gdict, "/MenuBar/EditMenu/Find", found_definition);
-  if (gdict->edit_actions)
-    gtk_action_group_set_sensitive (gdict->edit_actions, found_definition);
-}
-
 static void
 gdict_lookup_button_drag_cb (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
            GtkSelectionData *sd, guint info, guint t, gpointer data)
@@ -261,7 +237,6 @@ gdict_lookup_button_drag_cb (GtkWidget *widget, GdkDragContext *context, gint x,
 
   if (text) {
     gtk_entry_set_text (GTK_ENTRY (gdict->word_entry), text);
-    gnome_entry_prepend_history (GNOME_ENTRY (gdict->gnome_word_entry), 1, text);
     gdict_lookup_definition (gdict, text);
     g_free (text);
   }
@@ -408,7 +383,7 @@ gdict_find_cb (GtkAction *action, GtkWidget *widget)
 {
   GDictWindow *gdict = GDICT_WINDOW (widget);
 
-  if (gdict->find_dialog == NULL) {
+  if (!GTK_IS_WINDOW (gdict->find_dialog)) {
     GtkWidget *dialog;
     GtkWidget *hbox;
     GtkWidget *label;
@@ -447,10 +422,11 @@ gdict_find_cb (GtkAction *action, GtkWidget *widget)
     gdict->find_dialog = dialog;
     gdict->find_dialog_entry = entry;
   }
-  else
+  else {
     gtk_window_present (GTK_WINDOW (gdict->find_dialog));
+  }
 
-  if (gdict->find_text != NULL)
+  if (gdict->find_text)
     gtk_entry_set_text (GTK_ENTRY (gdict->find_dialog_entry), gdict->find_text);
 
   gtk_entry_select_region (GTK_ENTRY (gdict->find_dialog_entry), 0, -1);
@@ -461,7 +437,7 @@ static void
 find_next_cb (GtkAction *action, GtkWidget *widget)
 {
   GDictWindow *gdict = GDICT_WINDOW (widget);
-  if (gdict->find_text)
+  if (GTK_IS_ENTRY(gdict->find_text))
     gdict->search_from_beginning = !gdict_defbox_find (gdict->defbox, 
                                                        gdict->find_text, 
                                                        gdict->search_from_beginning);
@@ -479,6 +455,34 @@ gdict_preferences_cb (GtkAction *action, GtkWidget *widget)
   gtk_window_present (GTK_WINDOW (gdict->pref_dialog));
 }
 
+static void
+gdict_view_spellings_cb (GtkAction *action, GtkWidget *widget)
+{
+  GDictWindow *gdict = GDICT_WINDOW (widget);
+  
+  if (gdict->speller_visible) {
+    gtk_widget_hide (GTK_WIDGET(gdict->speller));
+    gdict->speller_visible = FALSE;
+  } else {
+    gtk_widget_show (GTK_WIDGET(gdict->speller));
+    gdict->speller_visible = TRUE;
+  }
+}
+
+static void
+gdict_zoom_in (GtkAction *action, GtkWidget *widget)
+{
+	GDictWindow *gdict = GDICT_WINDOW (widget);
+	gdict_defbox_zoom_in (gdict->defbox);
+}
+
+static void
+gdict_zoom_out (GtkAction *action, GtkWidget *widget)
+{
+	GDictWindow *gdict = GDICT_WINDOW (widget);
+	gdict_defbox_zoom_out (gdict->defbox);
+}
+
 static void
 gdict_help_contents_cb (GtkAction *action, GtkWidget *widget)
 {
@@ -501,77 +505,26 @@ gdict_help_contents_cb (GtkAction *action, GtkWidget *widget)
   }
 }
 
-static void
-gdict_web_search_cb (GtkAction *action, GtkWidget *widget, gpointer data)
-{
-  GDictWindow *gdict = GDICT_WINDOW (data);
-  GtkWidget *menu_item;
-  static int width = GDICT_DEFAULT_WIDTH, height = GDICT_DEFAULT_HEIGHT;
-  menu_item = gtk_ui_manager_get_widget (gdict->ui,
-              "/SearchTypeMenuBar/SearchTypeMenu/RegularSearch");
-  if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (menu_item))) {
-    gdict->search_type = SEARCH_TYPE_REGULAR;
-    gtk_widget_show (gdict->paned);
-    gtk_window_set_resizable (GTK_WINDOW (gdict), TRUE);
-    gtk_window_resize (GTK_WINDOW (gdict), width, height);
-  } else {
-    gdict->search_type = SEARCH_TYPE_WEB;
-    gtk_widget_hide (gdict->paned);
-    gtk_window_get_size (GTK_WINDOW (gdict), &width, &height);
-    gtk_window_set_resizable (GTK_WINDOW (gdict), FALSE);
-  }
-}
-
-static void
-gdict_zoom_in (GtkAction *action, GtkWidget *widget)
-{
-	GDictWindow *gdict = GDICT_WINDOW (widget);
-	gdict_defbox_zoom_in (gdict->defbox);
-}
-
-static void
-gdict_zoom_out (GtkAction *action, GtkWidget *widget)
-{
-	GDictWindow *gdict = GDICT_WINDOW (widget);
-	gdict_defbox_zoom_out (gdict->defbox);
-}
-
 static void
 gdict_about_cb (GtkAction *action, GtkWidget *widget)
 {
-  GDictWindow *gdict = GDICT_WINDOW (widget);
-  gdict_about (GTK_WINDOW(gdict));
+  gdict_about (widget);
 }
 
-static void
-gdict_view_spellings_cb (GtkAction *action, GtkWidget *widget)
+static gboolean
+gdict_window_delete (GtkWidget *widget, GdkEvent *event, gpointer data)
 {
-  GDictWindow *gdict = GDICT_WINDOW (widget);
-  
-  if (gdict->speller_visible) {
-    gtk_widget_hide (GTK_WIDGET(gdict->speller));
-    gdict->speller_visible = FALSE;
-  } else {
-    gtk_widget_show (GTK_WIDGET(gdict->speller));
-    gdict->speller_visible = TRUE;
-  }
-}
-
-static void
-gdict_window_close (GtkWidget *widget, GdkEvent *event, gpointer data)
-{
-  GDictWindow *gdict = data;
-  if (gdict->applet != NULL) 
+  GDictWindow *gdict = (GDictWindow *)data;
+  if (gdict->applet != NULL) {
     gtk_widget_hide (widget);
-  else {
-    gdict_windows = g_slist_remove (gdict_windows, GTK_WINDOW(gdict));    
-    dict_context_destroy (gdict->context); /* FIXME */
-    gdict_web_search_remove_all (gdict);
-    g_slist_free (gdict->web_search_sites);
-    gtk_widget_destroy (GTK_WIDGET(gdict));        
-    if (gdict_windows == NULL) 
-      gtk_main_quit ();
+    return TRUE;
   }
+ 
+  gdict_windows = g_slist_remove (gdict_windows, GTK_WINDOW(gdict));
+  if (gdict_windows == NULL) 
+    gtk_main_quit ();
+
+  return FALSE;
 }
 
 GtkWidget *
@@ -630,16 +583,6 @@ static const gchar ui_info[] =
 "  </menubar>"
 "</ui>";
 
-static const gchar search_type_ui[] = 
-"<ui>"
-"<menubar name='SearchTypeMenuBar'>"
-" <menu action='SearchTypeMenu'>"
-"   <menuitem action='RegularSearch'/>"
-"   <menuitem action='WebSearch'/>"
-" </menu>"
-"</menubar>"
-"</ui>";
-
 static const GtkActionEntry menu_entries[] = {
 
   /* Top */
@@ -677,14 +620,6 @@ static const GtkActionEntry menu_entries[] = {
     N_("About this application"), G_CALLBACK (gdict_about_cb) }
 };
 
-static const GtkRadioActionEntry search_type_entries[] = {
-  { "SearchTypeMenu", NULL, "Search Type"},
-  { "RegularSearch", NULL, N_("On a Dictionary Server"), NULL,
-    NULL, 1},
-  { "WebSearch", NULL, N_("On a Web Site"), NULL,
-    NULL, 2}
-};
-
 /* a separate group for items that change sensitivity */
 static const GtkActionEntry edit_entries[] = {
   { "Cut", GTK_STOCK_CUT, N_("Cu_t"), "<control>X",
@@ -704,61 +639,41 @@ static const GtkToggleActionEntry toggle_entries[] = {
 };
 
 static void
-menu_item_select_cb (GtkMenuItem *proxy, gpointer data)
+gdict_menu_item_set_sensitive (GDictWindow *gdict, char *path, gboolean sensitive)
 {
-  GDictWindow *gdict = data;
   GtkAction *action;
-  char *message;
 
-  action = g_object_get_data (G_OBJECT (proxy),  "gtk-action");
-  g_return_if_fail (action != NULL);
-
-  g_object_get (G_OBJECT (action), "tooltip", &message, NULL);
-  if (message)
-    {
-      gtk_statusbar_push (GTK_STATUSBAR (gdict->statusbar),
-                          gdict->tip_message_cid, message);
-      g_free (message);
-    }
+  g_return_if_fail (path);
+  action = gtk_ui_manager_get_action (gdict->ui, path);
+  gtk_action_set_sensitive (action, sensitive);
 }
 
-static void
-menu_item_deselect_cb (GtkMenuItem *proxy, gpointer data)
+void
+gdict_menus_set_sensitive (GDictWindow *gdict, gboolean found_definition)
 {
+  gdict_menu_item_set_sensitive (gdict, "/MenuBar/DictionaryMenu/Print", found_definition);
+  gdict_menu_item_set_sensitive (gdict, "/MenuBar/EditMenu/Find", found_definition);
+  if (gdict->edit_actions)
+    gtk_action_group_set_sensitive (gdict->edit_actions, found_definition);
+}
+
+static void 
+gdict_button_menu_changed (GtkComboBox *widget, gpointer data)
+{
+  static int width = GDICT_DEFAULT_WIDTH, height = GDICT_DEFAULT_HEIGHT;
   GDictWindow *gdict = data;
 
-  gtk_statusbar_pop (GTK_STATUSBAR (gdict->statusbar),
-                     gdict->tip_message_cid);
-}
-
-static void
-connect_proxy_cb (GtkUIManager *manager,
-                  GtkAction *action,
-                  GtkWidget *proxy,
-                  gpointer data)
-{
-  if (GTK_IS_MENU_ITEM (proxy))
-    {
-      g_signal_connect (proxy, "select",
-                  G_CALLBACK (menu_item_select_cb), data);
-      g_signal_connect (proxy, "deselect",
-                  G_CALLBACK (menu_item_deselect_cb), data);
-    }
-}
-
-static void
-disconnect_proxy_cb (GtkUIManager *manager,
-                     GtkAction *action,
-                     GtkWidget *proxy,
-                     gpointer data)
-{
-  if (GTK_IS_MENU_ITEM (proxy))
-    {
-      g_signal_handlers_disconnect_by_func (proxy,
-                  G_CALLBACK (menu_item_select_cb), data);
-      g_signal_handlers_disconnect_by_func (proxy,
-                  G_CALLBACK (menu_item_deselect_cb), data);
-    }
+  if (gtk_combo_box_get_active (GTK_COMBO_BOX (widget)) == 0) {
+    gdict->search_type = SEARCH_TYPE_REGULAR;
+    gtk_widget_show (gdict->paned);
+    gtk_window_set_resizable (GTK_WINDOW (gdict), TRUE);
+    gtk_window_resize (GTK_WINDOW (gdict), width, height);
+  } else {
+    gdict->search_type = SEARCH_TYPE_WEB;
+    gtk_widget_hide (gdict->paned);
+    gtk_window_get_size (GTK_WINDOW (gdict), &width, &height);
+    gtk_window_set_resizable (GTK_WINDOW (gdict), FALSE);
+  }
 }
 
 GtkWidget *
@@ -768,15 +683,13 @@ gdict_window_init (GDictWindow *gdict)
   GtkWidget *container;
   GtkWidget *hbox, *paned;
   GtkWidget *button;
-  GtkWidget *word_label;
   GtkWidget *scrolled;
+  GtkWidget *search_type_combo;
   GDictSpeller *speller;
   GtkWidget *window, *main_area;
-  GtkTooltips *tooltips;
-  GtkToolItem *menu_button;
   GtkUIManager *ui;
   GtkWidget *menu;
-  GtkWidget *gnome_word_entry, *word_entry;
+  GtkWidget *word_entry;
   GError *error = NULL;
   static GtkTargetEntry drop_targets [] = {
     { "UTF8_STRING", 0, 0 },
@@ -786,8 +699,6 @@ gdict_window_init (GDictWindow *gdict)
     { "STRING",     0, 0 }
   };
 
-  tooltips = gtk_tooltips_new ();
-
   window = GTK_WIDGET (gdict);
 
   gtk_window_set_default_size (GTK_WINDOW (window), GDICT_DEFAULT_WIDTH, GDICT_DEFAULT_HEIGHT);
@@ -806,15 +717,9 @@ gdict_window_init (GDictWindow *gdict)
               G_N_ELEMENTS (edit_entries), gdict);
   gtk_action_group_set_sensitive (gdict->edit_actions, FALSE);
 
-  gdict->search_type_actions = gtk_action_group_new ("SearchTypeActions");
-  gtk_action_group_add_radio_actions (gdict->search_type_actions, search_type_entries,
-              G_N_ELEMENTS (search_type_entries), 1, 
-              G_CALLBACK(gdict_web_search_cb), gdict);
-
   ui = gtk_ui_manager_new ();
   gtk_ui_manager_insert_action_group (ui, gdict->menu_actions, 0);
   gtk_ui_manager_insert_action_group (ui, gdict->edit_actions, 0);
-  gtk_ui_manager_insert_action_group (ui, gdict->search_type_actions, 0);
   gtk_window_add_accel_group (GTK_WINDOW (window), 
   gtk_ui_manager_get_accel_group (ui));
 
@@ -823,13 +728,6 @@ gdict_window_init (GDictWindow *gdict)
     g_error_free (error);
     return NULL;
   }
-  gtk_ui_manager_add_ui_from_string (ui, search_type_ui, -1, &error);
-
-  /* show tooltips in the statusbar */
-  g_signal_connect (ui, "connect_proxy",
-              G_CALLBACK (connect_proxy_cb), gdict);
-  g_signal_connect (ui, "disconnect_proxy",
-              G_CALLBACK (disconnect_proxy_cb), gdict);
 
   container = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), container);
@@ -838,49 +736,47 @@ gdict_window_init (GDictWindow *gdict)
   gtk_box_pack_start (GTK_BOX (container), menubar, FALSE, FALSE, 0);
 
   main_area = gtk_vbox_new (FALSE, 0);
-  gtk_box_set_spacing (GTK_BOX(main_area), 6);
-  gtk_container_set_border_width (GTK_CONTAINER (main_area), 6);
+  gtk_box_set_spacing (GTK_BOX(main_area), 0);
+  gtk_container_set_border_width (GTK_CONTAINER (main_area), 0);
   gtk_box_pack_start (GTK_BOX (container), main_area, TRUE, TRUE, 0);
 
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_box_set_spacing (GTK_BOX (hbox), 6);
   gtk_box_pack_start (GTK_BOX (main_area), hbox, FALSE, FALSE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
 
-  word_label = gtk_label_new_with_mnemonic (_("_Word:"));
-  gtk_box_pack_start (GTK_BOX (hbox), word_label, FALSE, FALSE, 0);
-
-  button = gdict_label_new_with_stock_image (_("_Look Up Word"), GTK_STOCK_FIND, NULL);
-  menu_button = gtk_menu_tool_button_new (button, NULL);
-  menu = gtk_widget_get_ancestor (gtk_ui_manager_get_widget (ui, "/SearchTypeMenuBar/SearchTypeMenu/RegularSearch"),
-          GTK_TYPE_MENU);
-  gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON(menu_button), GTK_WIDGET(menu));
-  gtk_box_pack_end (GTK_BOX (hbox), GTK_WIDGET(menu_button), FALSE, FALSE, 0);
-  gtk_drag_dest_set (GTK_WIDGET(menu_button), GTK_DEST_DEFAULT_ALL, drop_targets, 
-         G_N_ELEMENTS (drop_targets), GDK_ACTION_COPY | GDK_ACTION_MOVE);
-  gtk_tool_item_set_is_important (GTK_TOOL_ITEM (menu_button), TRUE);
-  g_signal_connect (G_OBJECT (menu_button), "clicked",
-        G_CALLBACK (gdict_lookup_cb), gdict);
+  button = gtk_button_new_with_mnemonic (_("_Look Up"));
+  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+  gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET(button), FALSE, FALSE, 0);
   g_signal_connect (G_OBJECT (button), "drag_data_received",
         G_CALLBACK (gdict_lookup_button_drag_cb), gdict);
+  g_signal_connect (G_OBJECT (button), "clicked",
+                    G_CALLBACK (gdict_lookup_cb), gdict);
+
+  search_type_combo = gtk_combo_box_new_text ();
+  gtk_combo_box_append_text (GTK_COMBO_BOX (search_type_combo), _("In Dictionary"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX (search_type_combo), _("On a Web Site"));
+  gtk_combo_box_set_active (GTK_COMBO_BOX (search_type_combo), 0);
+  gtk_box_pack_end (GTK_BOX (hbox), search_type_combo, FALSE, FALSE, 0);  
+  g_signal_connect (G_OBJECT (search_type_combo), "changed",
+                    G_CALLBACK (gdict_button_menu_changed), gdict);
+
+  word_entry = gdict_entry_new (gdict);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (GTK_BIN(button)->child), word_entry);
+  gtk_box_pack_start (GTK_BOX (hbox), word_entry, TRUE, TRUE, 0);
+  gtk_widget_grab_focus (word_entry);
 
-  gnome_word_entry = gnome_entry_new ("Wordentry");
-  gnome_entry_set_max_saved (GNOME_ENTRY (gnome_word_entry), 10);
-  gtk_box_pack_start (GTK_BOX (hbox), gnome_word_entry, TRUE, TRUE, 0);
-  word_entry = gnome_entry_gtk_entry(GNOME_ENTRY (gnome_word_entry));
   g_signal_connect (G_OBJECT(word_entry), "activate",
         G_CALLBACK (gdict_lookup_cb), gdict);
-  gtk_label_set_mnemonic_widget (GTK_LABEL (word_label), word_entry);
-  GTK_WIDGET_SET_FLAGS (word_entry, GTK_CAN_FOCUS);
-  gtk_widget_grab_focus (word_entry);
-  if (GTK_IS_ACCESSIBLE (gtk_widget_get_accessible (gnome_word_entry))) {
+
+  if (GTK_IS_ACCESSIBLE (gtk_widget_get_accessible (word_entry))) {
     gail_loaded = TRUE;
-    gdict_add_atk_namedesc (GTK_WIDGET(word_label), _("Word"), _("Word"));
-    gdict_add_atk_namedesc (gnome_word_entry, _("Word Entry"),
+    gdict_add_atk_namedesc (GTK_WIDGET(button), _("Word"), _("Word"));
+    gdict_add_atk_namedesc (word_entry, _("Word Entry"),
           _("Enter a Word or select one from the list below"));
-    gdict_add_atk_namedesc (word_entry, _("Word Entry"), NULL);
-    gdict_add_atk_relation (gnome_word_entry, GTK_WIDGET(word_label),
+    gdict_add_atk_relation (word_entry, GTK_WIDGET(button),
           ATK_RELATION_LABELLED_BY);
-    gdict_add_atk_namedesc (GTK_WIDGET(menu_button), NULL, _("Look Up for a Word"));
+    gdict_add_atk_namedesc (GTK_WIDGET(button), NULL, _("Look Up for a Word"));
   }
 
   paned = gtk_hpaned_new ();
@@ -905,13 +801,12 @@ gdict_window_init (GDictWindow *gdict)
   speller->strat = gdict->pref->dfl_strat;
 
   g_signal_connect (G_OBJECT (window), "delete_event",
-        G_CALLBACK (gdict_window_close), gdict);
+        G_CALLBACK (gdict_window_delete), gdict);
 
   gdict->speller = speller;
   gdict->paned = paned;
   gdict->ui = ui;
   gdict->word_entry = word_entry;
-  gdict->gnome_word_entry = gnome_word_entry;
   gdict->find_text = NULL;
   gdict->find_dialog = NULL;
   gdict->pref_dialog = NULL;
@@ -929,7 +824,13 @@ gdict_window_init (GDictWindow *gdict)
 static void
 gdict_window_finalize (GObject *object)
 {
-  GDictWindow *window = (GDictWindow *) object;
+  GDictWindow *gdict = (GDictWindow *) object;
+
+  dict_context_destroy (gdict->context); /* FIXME */
+  gdict_web_search_remove_all (gdict);
+  gdict_pref_destroy (gdict->pref);
+  g_slist_free (gdict->web_search_sites);
+
   parent_class->finalize (object);
 }
 
