@@ -185,7 +185,8 @@ gsearchtool_gconf_get_boolean (const gchar * key)
 }
 
 void
-gsearchtool_gconf_set_boolean (const gchar * key, const gboolean flag)
+gsearchtool_gconf_set_boolean (const gchar * key,
+                               const gboolean flag)
 {
 	GConfClient * client;
 	GError * error = NULL;
@@ -200,6 +201,57 @@ gsearchtool_gconf_set_boolean (const gchar * key, const gboolean flag)
 	gsearchtool_gconf_handle_error (&error);
 }
 
+void
+gsearchtool_gconf_add_dir (const gchar * dir)
+{
+	GConfClient * client;
+	GError * error = NULL;
+
+	g_return_if_fail (dir != NULL);
+
+	client = gsearchtool_gconf_client_get_global ();
+	g_return_if_fail (client != NULL);
+
+	gconf_client_add_dir (client,
+	                      dir,
+	                      GCONF_CLIENT_PRELOAD_RECURSIVE,
+	                      &error);
+
+	gsearchtool_gconf_handle_error (&error);
+}
+
+void
+gsearchtool_gconf_watch_key (const gchar * dir,
+                             const gchar * key, 
+                             GConfClientNotifyFunc callback,
+                             gpointer user_data)
+{
+	GConfClient * client;
+	GError * error = NULL;
+		
+	g_return_if_fail (key != NULL);
+	g_return_if_fail (dir != NULL);
+			
+	client = gsearchtool_gconf_client_get_global ();
+	g_return_if_fail (client != NULL);
+
+	gconf_client_add_dir (client,
+	                      dir,
+	                      GCONF_CLIENT_PRELOAD_NONE,
+	                      &error);
+
+	gsearchtool_gconf_handle_error (&error);
+
+	gconf_client_notify_add (client,
+	                         key,
+	                         callback,
+	                         user_data,
+	                         NULL,
+	                         &error);
+
+	gsearchtool_gconf_handle_error (&error);
+}
+
 /* START OF GENERIC GNOME-SEARCH-TOOL FUNCTIONS */
 
 gboolean 
@@ -986,7 +1038,8 @@ static gchar *
 gsearchtool_icon_lookup (GSearchWindow * gsearch,
                          const gchar * file, 
                          const gchar * mime,
-                         GnomeVFSFileInfo * file_info)
+                         GnomeVFSFileInfo * file_info,
+                         gboolean enable_thumbnails)
 {
 	GnomeThumbnailFactory * thumbnail_factory = NULL;
 	GnomeIconLookupFlags lookup_flags = GNOME_ICON_LOOKUP_FLAGS_NONE;
@@ -995,7 +1048,7 @@ gsearchtool_icon_lookup (GSearchWindow * gsearch,
 
 	uri = gnome_vfs_get_uri_from_local_path (file);
 
-	if (gsearch->show_thumbnails == TRUE) {
+	if ((enable_thumbnails == TRUE) && (gsearch->show_thumbnails == TRUE)) {
 		if ((strncmp (file_info->mime_type, "image/", 6) != 0) ||
 	    	    (file_info->size < gsearch->show_thumbnails_file_size_limit)) {
 		    	thumbnail_factory = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
@@ -1048,7 +1101,7 @@ get_file_pixbuf (GSearchWindow * gsearch,
 		icon_name = g_strdup (ICON_THEME_FIFO);
 	}
 	else {
-		icon_name = gsearchtool_icon_lookup (gsearch, file, file_info->mime_type, file_info);
+		icon_name = gsearchtool_icon_lookup (gsearch, file, file_info->mime_type, file_info, TRUE);
 	}
 	
 	pixbuf = (GdkPixbuf *) g_hash_table_lookup (gsearch->search_results_filename_hash_table, icon_name);
@@ -1062,6 +1115,10 @@ get_file_pixbuf (GSearchWindow * gsearch,
 		
 				if (strcmp (icon_name, file) == 0) {
 					pixbuf = gdk_pixbuf_new_from_file_at_scale (file, ICON_SIZE, ICON_SIZE, TRUE, NULL);
+					
+					if (pixbuf == NULL) {
+						icon_name = gsearchtool_icon_lookup (gsearch, file, file_info->mime_type, file_info, FALSE);
+					}
 				}
 			
 				if (pixbuf == NULL) {
@@ -1071,8 +1128,49 @@ get_file_pixbuf (GSearchWindow * gsearch,
 		}
 
 		if (pixbuf == NULL) {
-			pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), icon_name, 
-			                                   ICON_SIZE, 0, NULL);
+		
+			GdkPixbuf * thumbnail_pixbuf = NULL;
+			
+			thumbnail_pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), icon_name, 
+			                                             ICON_SIZE, 0, NULL);
+
+			if (thumbnail_pixbuf != NULL) {
+			
+				/*  The following is a workaround for bugzilla report #311318,                   */
+				/*  "gtk_icon_theme_load_icon () can return pixbufs larger than requested size". */
+				/*  Please see the url, http://bugzilla.gnome.org/show_bug.cgi?id=311318.        */
+			
+				if ((gdk_pixbuf_get_width (thumbnail_pixbuf) > ICON_SIZE) || 
+				    (gdk_pixbuf_get_height (thumbnail_pixbuf) > ICON_SIZE)) {
+
+					gfloat scale_factor_x = 1.0;
+					gfloat scale_factor_y = 1.0;
+					gint scale_x;
+					gint scale_y;
+
+					if (gdk_pixbuf_get_width (thumbnail_pixbuf) > ICON_SIZE) {
+						scale_factor_x = (gfloat) ICON_SIZE / (gfloat) gdk_pixbuf_get_width (thumbnail_pixbuf);
+					}
+					if (gdk_pixbuf_get_height (thumbnail_pixbuf) > ICON_SIZE) {
+						scale_factor_y = (gfloat) ICON_SIZE / (gfloat) gdk_pixbuf_get_height (thumbnail_pixbuf);
+					}
+				
+					if (gdk_pixbuf_get_width (thumbnail_pixbuf) > gdk_pixbuf_get_height (thumbnail_pixbuf)) {
+						scale_x = ICON_SIZE;
+						scale_y = (gint) (gdk_pixbuf_get_height (thumbnail_pixbuf) * scale_factor_x);
+					}
+					else {
+						scale_x = (gint) (gdk_pixbuf_get_width (thumbnail_pixbuf) * scale_factor_y);
+						scale_y = ICON_SIZE;
+					}    
+			    
+					pixbuf = gdk_pixbuf_scale_simple (thumbnail_pixbuf, scale_x, scale_y, GDK_INTERP_BILINEAR);
+					g_object_unref (thumbnail_pixbuf);
+				}
+				else {
+					pixbuf = thumbnail_pixbuf;
+				}
+			}
 		}
 		
 		if (pixbuf != NULL) {
