@@ -18,76 +18,20 @@
 
     ---------------------------------------------------------------------- */
 
-
-#include <config.h>
-#include <time.h>
-#include <gnome.h>
+#include <glib/gi18n.h>
+#include <stdlib.h>
 #include "gtk/gtk.h"
 #include "logview.h"
+#include "calendar.h"
 
-#define CALENDAR_WIDTH           180
-#define CALENDAR_HEIGHT          150
-
-#define SUNDAY                   0
-#define MONDAY                   1
-#define FRIDAY                   5
-#define SATURDAY                 6
-#define FEBRUARY                 1
-#define XSEP                     3
-#define CALLEFTMARGIN            5
-#define BASELINESKIP             4
-
-#define THISMONTH                1
-#define OTHERMONTH               2
-#define MARKEDDATE               3
-
-/*
- *       --------
- *       Typedefs
- *       --------
- */
-
-
-/*
- *    -------------------
- *    Function Prototypes
- *    -------------------
- */
-
-void CalendarMenu (GtkWidget * widget, gpointer user_data);
-void close_calendar (GtkWidget * widget, gpointer client_data);
-void set_scrollbar_size (int num_lines);
-void calendar_month_changed (GtkWidget *widget, gpointer data);
-void read_marked_dates (CalendarData *data);
-void calendar_month_changed (GtkWidget *widget, gpointer data);
-void calendar_day_selected (GtkWidget *widget, gpointer unused_data);
-void calendar_day_selected_double_click (GtkWidget *widget, gpointer data);
-CalendarData* init_calendar_data ();
-DateMark* find_prev_mark (CalendarData*);
-DateMark* find_next_mark (CalendarData*);
-DateMark* get_mark_from_month (CalendarData *data, gint month, gint year);
-DateMark *get_mark_from_date (CalendarData *, gint, gint, gint);
-void log_repaint (GtkWidget * canvas, GdkRectangle * area);
-
-
-
-/*
- *       ----------------
- *       Global variables
- *       ----------------
- */
-
-extern ConfigData *cfg;
-extern GdkGC *gc;
-extern Log *curlog, *loglist[];
-extern int numlogs, curlognum;
-extern char *month[12];
-extern GtkWidget *main_win_scrollbar;
-extern GnomeUIInfo view_menu[];
-GtkWidget *CalendarDialog = NULL;
-GtkWidget *CalendarWidget;
-int calendarvisible;
-
+static GtkTreePath *calendar_day_selected (GtkWidget *widget, LogviewWindow *window);
+static void calendar_day_selected_double_click (GtkWidget *widget, LogviewWindow *window);
+static void close_calendar (GtkWidget * widget, int arg, gpointer client_data);
+static DateMark* find_prev_mark (CalendarData*);
+static DateMark* find_next_mark (CalendarData*);
+static DateMark* get_mark_from_month (CalendarData *data, gint month, gint year);
+static DateMark *get_mark_from_date (CalendarData *, gint, gint, gint);
+static void calendar_month_changed (GtkWidget *widget, LogviewWindow *window);
 
 /* ----------------------------------------------------------------------
    NAME:          CalendarMenu
@@ -95,66 +39,34 @@ int calendarvisible;
    ---------------------------------------------------------------------- */
 
 void
-CalendarMenu (GtkWidget * widget, gpointer user_data)
+CalendarMenu (LogviewWindow *window)
 {
-   GtkCalendar *calendar;
-   GtkWidget *vbox;
+	PangoFontDescription *fontdesc;
+	PangoContext *context;
+	GtkCalendar *calendar;
+	int size;
+	
+	calendar = (GtkCalendar *)gtk_calendar_new();
+	
+	g_signal_connect (G_OBJECT (calendar), "month_changed",
+										G_CALLBACK (calendar_month_changed),
+										window);
+	g_signal_connect (G_OBJECT (calendar), "day_selected",
+										G_CALLBACK (calendar_day_selected),
+										window);
+	g_signal_connect (G_OBJECT (calendar), "day_selected_double_click",
+										G_CALLBACK (calendar_day_selected_double_click),
+										window);
+	window->calendar = GTK_WIDGET (calendar);
+	window->calendar_visible = TRUE;
 
-   if (curlog == NULL || calendarvisible)
-      return;
-
-   if (CalendarDialog == NULL)
-   {
-      CalendarDialog = gtk_dialog_new ();
-
-      gtk_window_set_title (GTK_WINDOW (CalendarDialog), _("Calendar"));
-      gtk_window_set_resizable (GTK_WINDOW (CalendarDialog), FALSE);
-      gtk_dialog_set_has_separator (GTK_DIALOG(CalendarDialog), FALSE);
-      gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (CalendarDialog)), 5);
-
-      gtk_dialog_add_button (GTK_DIALOG (CalendarDialog), 
-			     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
-      g_signal_connect (G_OBJECT (CalendarDialog), "response",
-		        G_CALLBACK (close_calendar),
-			&CalendarDialog);
-      g_signal_connect (G_OBJECT (CalendarDialog), "destroy",
-			G_CALLBACK (close_calendar),
-			NULL);
-      g_signal_connect (G_OBJECT (CalendarDialog), "delete_event",
-			G_CALLBACK (gtk_true),
-			NULL);
-
-      vbox = gtk_vbox_new (FALSE, 6);
-
-      gtk_box_pack_start (GTK_BOX (GTK_DIALOG (CalendarDialog)->vbox), vbox, 
-			  TRUE, TRUE, 0);
-      gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
-
-      calendar = (GtkCalendar *)gtk_calendar_new();
-
-      gtk_signal_connect (GTK_OBJECT (calendar), "month_changed",
-			  GTK_SIGNAL_FUNC (calendar_month_changed),
-			  NULL);
-      gtk_signal_connect (GTK_OBJECT (calendar), "day_selected",
-			  GTK_SIGNAL_FUNC (calendar_day_selected),
-			  NULL);
-      gtk_signal_connect (GTK_OBJECT (calendar), "day_selected_double_click",
-			  GTK_SIGNAL_FUNC (calendar_day_selected_double_click),
-			  NULL);
-      gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (calendar), TRUE, TRUE, 0);
-      gtk_widget_show (GTK_WIDGET (calendar));
-
-      gtk_dialog_set_default_response (GTK_DIALOG (CalendarDialog), GTK_RESPONSE_CLOSE); 
-   
-      gtk_widget_show (vbox);
-      
-      CalendarWidget = GTK_WIDGET (calendar);
-      init_calendar_data ();
-   }
-   calendarvisible = TRUE;
-
-   gtk_widget_show (CalendarDialog);
+	context = gtk_widget_get_pango_context (GTK_WIDGET(calendar));
+	fontdesc = pango_context_get_font_description (context);
+	size = pango_font_description_get_size (fontdesc) / PANGO_SCALE;
+	pango_font_description_set_size (fontdesc, (size-2)*PANGO_SCALE);
+	gtk_widget_modify_font (GTK_WIDGET (calendar), fontdesc);
 
+	gtk_widget_show (GTK_WIDGET (calendar));
 }
 
 /* ----------------------------------------------------------------------
@@ -164,14 +76,14 @@ CalendarMenu (GtkWidget * widget, gpointer user_data)
    ---------------------------------------------------------------------- */
 
 void
-read_marked_dates (CalendarData *data)
+read_marked_dates (CalendarData *data, LogviewWindow *window)
 {
   GtkCalendar *calendar;
   DateMark *mark;
   gint day, month, year;
 
   g_return_if_fail (data);
-  calendar = GTK_CALENDAR (CalendarWidget);
+  calendar = GTK_CALENDAR (window->calendar);
   g_return_if_fail (calendar);
 
   gtk_calendar_clear_marks (calendar);
@@ -184,8 +96,8 @@ read_marked_dates (CalendarData *data)
   while (mark)
     {
       if (mark->fulldate.tm_mon != month || 
-	  mark->fulldate.tm_year != year)
-	break;
+					mark->fulldate.tm_year != year)
+				break;
       gtk_calendar_mark_day (calendar, mark->fulldate.tm_mday);
       mark = mark->next;
     }
@@ -200,30 +112,33 @@ read_marked_dates (CalendarData *data)
    ---------------------------------------------------------------------- */
 
 CalendarData*
-init_calendar_data ()
+init_calendar_data (LogviewWindow *window)
 {
    CalendarData *data;
+	 
+	 if (window->curlog == NULL)
+		 return NULL;
 
-   data = curlog->caldata;
+   data = window->curlog->caldata;
    if (data == NULL)
      data = (CalendarData*) malloc (sizeof (CalendarData));
 
-   if (data)
-     {
-       data->curmonthmark = curlog->lstats.firstmark;
-       curlog->caldata = data;
-
-#if 0
-       /* Move mark to first marked date in this month */
-       data->curmonthmark = 
-	 get_mark_from_month (data, curlog->curmark->fulldate.tm_mon,
-			      curlog->curmark->fulldate.tm_year);
-#endif
-       
-       /* signal a redraw event */
-       gtk_signal_emit_by_name (GTK_OBJECT (CalendarWidget), "month_changed");
-       
-     }
+   if (data) {
+		 DateMark *mark;
+		 data->curmonthmark = window->curlog->lstats.firstmark;
+		 window->curlog->caldata = data;
+		 
+		 mark = data->curmonthmark;
+		 if (mark) {
+			 gtk_calendar_select_month (GTK_CALENDAR(window->calendar), 
+																	mark->fulldate.tm_mon,
+																	mark->fulldate.tm_year + 1900);
+			 gtk_calendar_select_day (GTK_CALENDAR(window->calendar), 
+																mark->fulldate.tm_mday);
+		 }
+		 
+		 read_marked_dates (data, window);       
+	 }
 
    return data;
 }
@@ -233,18 +148,21 @@ init_calendar_data ()
    DESCRIPTION:   User changed the month in the calendar.
    ---------------------------------------------------------------------- */
 
-void
-calendar_month_changed (GtkWidget *widget, gpointer unused_data)
+static void
+calendar_month_changed (GtkWidget *widget, LogviewWindow *window)
 {
   GtkCalendar *calendar;
   CalendarData *data;
   DateMark *mark;
   gint day, month, year;
 
-  calendar = GTK_CALENDAR (CalendarWidget);
+	if (window->curlog == NULL)
+		return;
+
+  calendar = GTK_CALENDAR (window->calendar);
   g_return_if_fail (calendar);
 
-  data = curlog->caldata;
+  data = window->curlog->caldata;
   g_return_if_fail (data);
 
   /* Get current date */
@@ -256,7 +174,7 @@ calendar_month_changed (GtkWidget *widget, gpointer unused_data)
   mark = get_mark_from_month (data, month, year);
   if (mark)
     data->curmonthmark = mark;
-  read_marked_dates (data);
+  read_marked_dates (data, window);
 }
 
 
@@ -265,44 +183,24 @@ calendar_month_changed (GtkWidget *widget, gpointer unused_data)
    DESCRIPTION:   User clicked on a calendar entry
    ---------------------------------------------------------------------- */
 
-void
-calendar_day_selected (GtkWidget *widget, gpointer unused_data)
+static GtkTreePath *
+calendar_day_selected (GtkWidget *widget, LogviewWindow *window)
 {
-#ifdef FIXME
-  /* need to update the calander days with with the ones in logfile */
-
-  GtkCalendar *calendar;
-  CalendarData *data;
+  /* find the selected day in the current logfile */
   gint day, month, year;
+  GtkTreePath *path;
 
-  calendar = GTK_CALENDAR (CalendarWidget);
-  gtk_calendar_get_date (calendar, &year, &month, &day);
-  /* This is Y2K compatible but internally we keep track of years
-     starting from 1900. This is because of Unix and not because 
-     I like it!! */
-  year -= 1900;
-  g_return_if_fail (calendar);
+	if (window->curlog == NULL)
+		return NULL;
 
-  data = curlog->caldata;
-  g_return_if_fail (data);
-  
-  /* TODO This is an ugly HACK until I add this function into
-     the widget!!!!!!
-  */
-  /* Dates go from 0-30 in this array!!! I really need to
-     add the function gtk_calendar_day_marked (day) */
-  if (calendar->marked_date[day-1])
-    {
-      curlog->curmark = get_mark_from_date (data, day, month, year);
-      MoveToMark (curlog);
-      curlog->firstline = 0;
-      curlog->ln = curlog->curmark->ln;
-      gtk_adjustment_set_value ( GTK_RANGE(main_win_scrollbar)->adjustment,
-				 curlog->ln);
-      /* set_scrollbar_size (curlog->lstats.numlines); */
-      log_repaint(NULL, NULL);
-    }
-#endif  
+  gtk_calendar_get_date (GTK_CALENDAR (window->calendar), &year, &month, &day);
+
+  path = g_hash_table_lookup (window->curlog->date_headers,
+                DATEHASH (month, day));
+  if (path != NULL)
+		gtk_tree_view_set_cursor (GTK_TREE_VIEW(window->view), path, NULL, FALSE);
+
+	return path;
 }
 
 /* ----------------------------------------------------------------------
@@ -311,9 +209,12 @@ calendar_day_selected (GtkWidget *widget, gpointer unused_data)
    ---------------------------------------------------------------------- */
 
 void
-calendar_day_selected_double_click (GtkWidget *widget, gpointer data)
+calendar_day_selected_double_click (GtkWidget *widget, LogviewWindow *window)
 {
-  calendar_day_selected(widget, data);
+	GtkTreePath *path;
+  path = calendar_day_selected(widget, window);
+	if (path)
+		gtk_tree_view_expand_row (GTK_TREE_VIEW (window->view), path, FALSE);
 }
 
 /* ----------------------------------------------------------------------
@@ -322,13 +223,14 @@ calendar_day_selected_double_click (GtkWidget *widget, gpointer data)
    ---------------------------------------------------------------------- */
 
 void
-close_calendar (GtkWidget *widget, gpointer client_data)
+close_calendar (GtkWidget *widget, int arg, gpointer data)
 {
-   if (calendarvisible) {
-      gtk_widget_hide (CalendarDialog);
-      gtk_check_menu_item_set_active  (GTK_CHECK_MENU_ITEM (view_menu[0].widget), FALSE);
+   LogviewWindow *window = LOGVIEW_WINDOW (data);
+   if (window->calendar_visible) {
+	   GtkAction *action = gtk_ui_manager_get_action (window->ui_manager, "/LogviewMenu/ViewMenu/ShowCalendar");
+	   gtk_action_activate (action);
    }
-   calendarvisible = FALSE;
+   window->calendar_visible = FALSE;
 }
 
 
@@ -338,7 +240,7 @@ close_calendar (GtkWidget *widget, gpointer client_data)
                 entry that month.
    ---------------------------------------------------------------------- */
 
-DateMark *
+static DateMark *
 get_mark_from_date (CalendarData *data, gint day, gint month, gint year)
 {
   DateMark *mark;
@@ -362,7 +264,7 @@ get_mark_from_date (CalendarData *data, gint day, gint month, gint year)
                 entry that month.
    ---------------------------------------------------------------------- */
 
-DateMark *
+static DateMark *
 get_mark_from_month (CalendarData *data, gint month, gint year)
 {
    DateMark *mark = data->curmonthmark;
@@ -402,7 +304,7 @@ get_mark_from_month (CalendarData *data, gint month, gint year)
    DESCRIPTION:   Returns the previous mark with a different month.
    ---------------------------------------------------------------------- */
 
-DateMark *
+static DateMark *
 find_prev_mark (CalendarData *data)
 {
    DateMark *mark = data->curmonthmark;
@@ -431,7 +333,7 @@ find_prev_mark (CalendarData *data)
    DESCRIPTION:   Returns the next mark with a different month.
    ---------------------------------------------------------------------- */
 
-DateMark *
+static DateMark *
 find_next_mark (CalendarData *data)
 {
    DateMark *mark = data->curmonthmark;
