@@ -111,14 +111,14 @@ mon_update_display (LogviewWindow *window)
 
 	   for (i=nlines; i >= (nlines-nlines_to_show); --i) {
 		   if (buffer_lines[i] != NULL) {
-			   ParseLine (buffer_lines[i], line);
+			   ParseLine (buffer_lines[i], line, TRUE);
 			   if (line->date > 0) {
 				   mon_format_line (buffer, sizeof (buffer), line);
 				   gtk_list_store_insert_before (list, &iter, parent_pointer);
 				   if (bold)
-					   marked_line = g_strdup_printf ("<b>%s</b>", buffer);
+					   marked_line = g_markup_printf_escaped ("<b>%s</b>", buffer);
 				   else
-					   marked_line = g_strdup (buffer);
+					   marked_line = g_markup_printf_escaped ("%s", buffer);
 				   gtk_list_store_set (list, &iter, 0, marked_line, -1);
 				   parent = iter;
 				   g_free (marked_line);
